C *******************PROGRAM DIECAST************************************
C DIECAST OCEAN MODEL: idealized density current               May, 2004
C This adaptation from the full DieCAST family of models simulates density
C current dynamics in an idealized density current setup patterned after
C the East Greenland Current interaction with the North Atlantic Gyre.
C **********************************************************************
C This is similar to the 3-d Beckmann and Doscher (1997) problem.

Contrary to claims that z-level models are extremely deficient in density
C current modeling (e.g., Beckmann and Doscher, 1997; Ezer and Mellor,
C 2004), that are based on results from z-level models having less accurate
C and robust numerics, the results of this z-level DieCAST model adaptation
C show good density current behavior.
 
C MAIN PROGRAM
C Controls calculation, performs diagnostics, and saves graphics data.  
C Graphics and computer animation is done by postprocessors
 
C *** User-defined resolution parameters ***
      INCLUDE 'resolution.h'

C =============================================================
C Horizontal resolution parameters are I0,I1,I2,I3, J0,J1,J2,J3
C Vertical resolution parameters are K0,K1,K2
C NB0,NB1 are dimension parameters for SEVP elliptic solver
C =============================================================

C Derived resolution parameters
      PARAMETER(IJ0=I0+J0,I1=I0-1,I2=I0-2,J1=J0-1,J2=J0-2,
     1 K1=K0-1,K2=K0-2,NB1=NB0-1)

C Double precision SEVP elliptic solver arrays
      REAL*8 RINV,RINV1,DUM0,DUM1,DUM2,X,H
C Lower precision logical depth and masking arrays
      INTEGER*2 KB,IU0,IV0,IN,IU,IV,IW

C "A" grid arrays
      COMMON RHO(I0,J0,K1),PX(I0,J0),PY(I0,J0),
     1 U1(I0,J0,K1),U2(I0,J0,K1),V1(I0,J0,K1),V2(I0,J0,K1),
     2 T1(I0,J0,K1),T2(I0,J0,K1),P0(I0,J0),P(I0,J0,K1),
     3 ULF(I0,J0,K1),VLF(I0,J0,K1),TLF(I0,J0,K1),
     4 EV(I2,J2,K2),HV(I2,J2,K2),DAVG(K1),F(J2),SUMIN(K1)

C initial conditions
      COMMON/TINIT/T0(I0,J0,K1)

C "C" grid arrays
      COMMON/CGRID/U(I0,J0,K1),V(I0,J1,K1),W(I0,J0,K0)

C User-defined scalar control parameters
      COMMON/CONTRO/
     1 DT,TLZ,B,G,DM0,DE0,DMZ0,RZMX,TAU,DRAG,FLTW,AV,DAODT,TAUDT,
     2 TAUDTN,AROUT,KTRM,M2,M6,MVI,MXIT,ISAV,
     3 MXSAV,LRSTRT,LOPEN,LSURF,LTURB,LWIND,LHEAT,LSAVE,LMOVI
 
C Vertical grid arrays
C "Z" array contains layer center AND interface depths
      COMMON/ZFS/Z(K0+K1),ODZ(K1),ODZW(K0)

C Logical depth and masking arrays
C logical depth array "KB" is an array that is derived from user-
C specified depths in the preprocessor PREP/prep.f
      COMMON/BATHY/KB(I0,J0),IU0(I0,J0),IV0(I0,J0),
     1 IN(I0,J0,K1),IU(I0,J0,K1),IV(I0,J1,K1),IW(I0,J0,K0)

C SEVP elliptic solver inverse and influence coefficient arrays
C SEVP subregion boundary array "IE" is a user-defined array
C Constraints on IE definition are based on roundoff error properties
C of the efficient "Stabilized Error Vector Propagation" elliptic solver
C used to derive a rigid-lid pressure that maintains incompressibility
C This is a main feature of the DieCAST family of ocean models, and has
C been extended to ultra-high resolution grids by domain decomposition.
C Reference: "Elliptic Marching Methods and Domain Decomposition", by
C Pat Roache (CRC publishers)
      COMMON/SEVP/RINV(I2,I2,NB0),RINV1(I2,I2,NB1),DUM0(I2,NB1),
     1 DUM1(I2),DUM2(I2),X(I0,J0),H(I0,J0),AL(I2,J2),AB(I2,J2),
     2 AC(I2,J2),AR(I2,J2),AT(I2,J2),S(I2,J2),IE(NB0)

C Derived scalars
      COMMON/SCA/ODT,PRN,DAYS,ORZMX,OFLTW,ITF,IT0
      COMMON/METRICS/
     1 Y(J0),YV(J0),
     2 CS(J0),CSV(J1),OCS(J0),OCSV(J1),
     3 DX(J0),DXV(J1),ODX(J0),ODXV(J1),
     4 DY(J0),DYV(J1),ODY(J0),ODYV(J1)
      COMMON/SURF/TSURF(I2,J2)

C bottom time average arrays
      COMMON/TIMAVG/UBAR(I0,J0),VBAR(I0,J0),WBAR(I0,J0),TBAR(I0,J0)

C scratch arrays
      COMMON/KLIMAT/SCR(I0,J0,K1),SC1(IJ0),SC2(IJ0)

C ----------------------
C OPEN OUTPUT DATA FILES
C ----------------------
C Run history data
      OPEN(14,file='TR')
C Restart data
      OPEN(19,file='SV',form='unformatted')
C Postprocessing graphics data:
C X-Y, X-Z and Y-Z cross-section data files in subdirectories:
C X-Y PLOTS: XYPLOTS/DATA (LOGICAL UNIT 30)
C X-Z PLOTS: XZPLOTS/DATA (LOGICAL UNIT 31)
C Y-Z PLOTS: YZPLOTS/DATA (LOGICAL UNIT 32)
      OPEN(30,file='XYPLOTS/DATA',form='unformatted')
      OPEN(31,file='XZPLOTS/DATA',form='unformatted')
      OPEN(32,file='YZPLOTS/DATA',form='unformatted')

C Stop-run file ('LSTOP' is flag to stop run)
      OPEN(42,file='stoprun')
      REWIND 14
      REWIND 19
      REWIND 30
      REWIND 31
      REWIND 32
      REWIND 42
      LSTOP=0
      WRITE(42,416) LSTOP

C --------------------------------------------------
C INITIALIZE DERIVED SCALARS AND ALL NON-ZERO FIELDS
C --------------------------------------------------
      CALL INITFS
      IF (ITF.GE.MXIT) STOP

      NSTPS=0
      NDAYS=0
      IT0=ITF

C save initial temperature
      DO 99 K=1,K1
      DO 99 J=1,J0
      DO 99 I=1,I0
 99   T0(I,J,K)=T1(I,J,K)

C ------------------------------
C MAIN TIME INTEGRATION LOOP 100
C ------------------------------
 100  ITF=ITF+1
      DAYS=ITF/DAODT

C FS marches one time step each time it is called
C ===========
      CALL FS
C ===========

C Monitor solution progress in detail and check for possible instability
C NOTE: DIECAST has been found to ALWAYS BE STABLE unless DT is too large
C or user-specified i.c.s or b.c.s are UNPHYSICAL)
      TMP=0.
      DO 103 J=2,J1
      DO 103 I=2,I1
 103  TMP=MAX(TMP,U2(I,J,KB(I,J))**2,V2(I,J,KB(I,J))**2)
      VMAX=SQRT(TMP)
      CALL RANGE(V2,SC1,SC2,IN,I0,2,2,I1,J1,ILO,JLO,IHI,JHI,VLO,VHI)
      IF (MOD(ITF,MVI).NE.0) GO TO 108
      TEMP=DAYS/360.
      WRITE(*,107) ITF,TEMP,ILO,JLO,VLO,IHI,JHI,VHI,VMAX
      WRITE(14,107) ITF,TEMP,ILO,JLO,VLO,IHI,JHI,VHI,VMAX
 107  FORMAT('at itf=',I6,', year',F7.2,/
     1 '(ILO,JLO,VLO,IHI,JHI,VHI)=(',2I4,1X,F9.4,2I4,1X,F9.4,')'/
     2 'max bbl vel=',1PE8.1)
 108  IF ((VHI-VLO).LT.1000..AND.VMAX.LT.300.) GO TO 110
      WRITE(14,107) ITF,TEMP,ILO,JLO,VLO,IHI,JHI,VHI,VMAX
      WRITE(14,109) ITF
      WRITE(*,107) ITF,TEMP,ILO,JLO,VLO,IHI,JHI,VHI,VMAX
      WRITE(*,109) ITF
 109  FORMAT('STOP. UNIX-pectedly large velocity at ITF=',I6)
      CALL FSPLTS(MXIT,0)
      stop0

 110  IF (MOD(ITF,MVI).NE.0) GO TO 400
C high frequency bottom plume data for animation and/or plots
      DO 112 J=2,J1
      DO 112 I=2,I1
      SCR(I,J,2)=U2(I,J,KB(I,J))
      SCR(I,J,3)=V2(I,J,KB(I,J))
 112  SCR(I,J,4)=T0(I,J,KB(I,J))-T2(I,J,KB(I,J))
      CALL XYPLOT('R','bbl temperature anomaly & vel ',SCR(1,1,4),1,1)
      CALL XYPLOT('u','bbl longitudinal vel. (cm/sec)',SCR(1,1,2),1,1)
      CALL XYPLOT('v','bbl latitudinal vel. (cm/sec) ',SCR(1,1,3),1,1)
      J=J1
      CALL XZPLOT('T','temperature (degrees C)       ',T2(1,J,1),J,J)

C accumulate time average data for sequential spans
      ITFDAY=DAODT
C 5-day spans
C     IF (MOD(ITF,5*ITFDAY).NE.1) GO TO 119
C 1-day spans
      IF (MOD(ITF,ITFDAY).NE.1) GO TO 119
      CALL SETR(UBAR,4*I0*J0,0.)
      AV=0.
 119  AV=AV+1.
      WT=1./AV
      OWT=1.-WT
      TMP=24.*36.
      DO 120 J=2,J1
      DO 120 I=2,I1
      UBAR(I,J)=OWT*UBAR(I,J)+WT*U2(I,J,KB(I,J))
      VBAR(I,J)=OWT*VBAR(I,J)+WT*V2(I,J,KB(I,J))
C vertical velocity at top of bottom cell
      WBAR(I,J)=OWT*WBAR(I,J)+WT*TMP*W(I,J,KB(I,J)-1)
 120  TBAR(I,J)=OWT*TBAR(I,J)+WT*(T0(I,J,KB(I,J))-T2(I,J,KB(I,J)))
      IF (MOD(ITF,5*ITFDAY).NE.0) GO TO 400
      CALL XYPLOT('Q','recent time avg bbl Tanom & V ',TBAR,1,1)
      CALL XYPLOT('u','bbl longitudinal vel. (cm/sec)',UBAR,1,1)
      CALL XYPLOT('v','bbl latitudinal vel. (cm/sec) ',VBAR,1,1)
      CALL XYPLOT('W','recent time avg bbl W(m/day)  ',WBAR,1,1)

 400  IF (MOD(ITF,ISAV).NE.0.AND.ITF.NE.MXIT) GO TO 402
      ISAV=MIN0(2*ISAV,MXSAV)

Contour and vector plots
      I=1
      IF (ITF.EQ.MXIT) I=0
      CALL FSPLTS(MXIT,I)
      CALL FLUSH(30)
      CALL FLUSH(31)
      CALL FLUSH(32)
 
 402  IF (LSAVE.NE.1.OR.(MOD(ITF,M2).NE.0.AND.ITF.NE.MXIT)) GO TO 410
 
C Save restart data
 403  REWIND 19
      WRITE(19) ITF,DAYS,AV,(X(I,2),I=2,I1),U1,U2,V1,V2,
     1 T1,T2,P,ULF,VLF,TLF,U,V,W,EV,HV
      NXTSAV=ITF+M2
      WRITE(14,404) ITF
      WRITE(*,404) ITF
 404  FORMAT('SAVE AT ITF= ',I6)
 410  IF (MOD(ITF,10).NE.0) GO TO 420
      IF (LSTOP.NE.0) stop1
      REWIND 42
      READ(42,416) LSTOP
 416  FORMAT(I6) 
      IF (LSTOP.GT.1) MXIT=LSTOP
      IF (LSTOP.GT.1) LSTOP=0
      IF (LSTOP.EQ.1) GO TO 403
 420  IF (MOD(ITF,M2).EQ.0) WRITE(14,425) ITF,NXTSAV
 425  FORMAT(' ITF=',I6,', next save at ITF=',I6)
      IF (ITF.LT.MXIT) GO TO 100
      END
**************************************************
C MAIN DYNAMICS PROGRAM AND ASSOCIATED SUBROUTINES
**************************************************
      SUBROUTINE FS
C ----------------------------------------------------------------------
C FS is a wind stress and buoyancy driven free stream model, with
C rigid top, variable DZ, and bottom topography.
C FS resolves bottom features using a staircase approximation.
C EXACT conservation is satisfied.
C Bottom fluxes are specified as SOURCES when coupled to TS (thin-shell
C submodel). W is positive downward. W(I,J,1) is at free surface.
C W(I,J,KB(I,J)+1), where 0.LE.KB.LE.K1, is at bottom
C ----------------------------------------------------------------------
      INCLUDE 'resolution.h'
      PARAMETER(IJ0=I0+J0,I1=I0-1,I2=I0-2,I3=I0-3,J1=J0-1,
     1 J2=J0-2,J3=J0-3,K1=K0-1,K2=K0-2,NB1=NB0-1)
      REAL*8 RINV,RINV1,DUM0,DUM1,DUM2,X,H
      REAL*8 WFACE,UFACE,VFACE
      INTEGER*2 KB,IU0,IV0,IN,IU,IV,IW
      COMMON RHO(I0,J0,K1),PX(I0,J0),PY(I0,J0),
     1 U1(I0,J0,K1),U2(I0,J0,K1),V1(I0,J0,K1),V2(I0,J0,K1),
     2 T1(I0,J0,K1),T2(I0,J0,K1),P0(I0,J0),P(I0,J0,K1),
     3 ULF(I0,J0,K1),VLF(I0,J0,K1),TLF(I0,J0,K1),
     4 EV(I2,J2,K2),HV(I2,J2,K2),DAVG(K1),F(J2),SUMIN(K1)
C Pressure 4th-order-accurate face average arrays
      COMMON/PGG/WFACE(I2,J2,K0),UFACE(I1,J2),VFACE(I2,J1)
      COMMON/CGRID/U(I0,J0,K1),V(I0,J1,K1),W(I0,J0,K0)
      COMMON/CONTRO/
     1 DT,TLZ,B,G,DM0,DE0,DMZ0,RZMX,TAU,DRAG,FLTW,AV,DAODT,TAUDT,
     2 TAUDTN,AROUT,KTRM,M2,M6,MVI,MXIT,ISAV,
     3 MXSAV,LRSTRT,LOPEN,LSURF,LTURB,LWIND,LHEAT,LSAVE,LMOVI
      COMMON/FLX/
     1 UX(I1,J2),UY(I2,J1),UZ(I2,J2,2),VX(I1,J2),VY(I2,J1),VZ(I2,J2,2),
     2 TX(I1,J2),TY(I2,J1),TZ(I2,J2,2)
      COMMON/SURF/TSURF(I2,J2)
      COMMON/KLIMAT/SCR(I0,J0,K1),SC1(IJ0),SC2(IJ0)
C Northern and southern sponge layer Levitus Climatology
      COMMON/SCA/ODT,PRN,DAYS,ORZMX,OFLTW,ITF,IT0
      COMMON/METRICS/
     1 Y(J0),YV(J0),
     2 CS(J0),CSV(J1),OCS(J0),OCSV(J1),
     3 DX(J0),DXV(J1),ODX(J0),ODXV(J1),
     4 DY(J0),DYV(J1),ODY(J0),ODYV(J1)
      COMMON/SEVP/RINV(I2,I2,NB0),RINV1(I2,I2,NB1),DUM0(I2,NB1),
     1 DUM1(I2),DUM2(I2),X(I0,J0),H(I0,J0),AL(I2,J2),AB(I2,J2),
     2 AC(I2,J2),AR(I2,J2),AT(I2,J2),S(I2,J2),IE(NB0)
      COMMON/BATHY/KB(I0,J0),IU0(I0,J0),IV0(I0,J0),
     1 IN(I0,J0,K1),IU(I0,J0,K1),IV(I0,J1,K1),IW(I0,J0,K0)
C Z IS DISTANCE FROM TOP OCEAN BOUNDARY, INCREASING WITH INCREASING K
      COMMON/ZFS/Z(K0+K1),ODZ(K1),ODZW(K0)
      O12=1./12
      O24=1./24

C ==========================
C Set vertical diffusivities
C ==========================
C Set background vertical viscosity and diffusivities to DMZ0 and
C add numerical contribution according to directed cell Reynolds number.
c     DO 150 K=1,K2
c     DO 150 J=1,J2
c     DO 150 I=1,I2
c     HV(I,J,K)=(DMZ0*ODZW(K+1)+ORZMX*ABS(W(I+1,J+1,K+1)))
c    1 *IW(I+1,J+1,K+1)
c150  EV(I,J,K)=HV(I,J,K)

C Modified Pacanowski and Philander approach
      CALL PP82

C ======================================================
C Hydrostatic equation with static balance subracted out
C ======================================================
 
C BD97 sill problem
      DO 92 K=1,K1
      DO 92 J=2,J1
      DO 92 I=2,I1
C linear EOS
 92   RHO(I,J,K)=.0002*(20.-T2(I,J,K))
Calculate face averaged pressure at bottom of each control volume
C at w-levels: WFACE(I2,J2,K0)
      DO 99 J=1,J2
      DO 99 I=1,I2
 99   WFACE(I,J,1)=P0(I+1,J+1)

C     REDG=G/(1.+100.*EXP(-0.5*DAYS))
      DO 100 K=1,K1
      L=K+1
C     TMP=.5*REDG/ODZW(L)
      TMP=G/ODZ(K)
      DO 100 J=1,J2
      DO 100 I=1,I2
 100  WFACE(I,J,L)=WFACE(I,J,K)+TMP*RHO(I+1,J+1,K)

Calculate control volume averaged pressure from face averages
C 2nd-order-accurate in top and bottom layer
      DO 102 J=1,J2
      DO 102 I=1,I2
      P(I+1,J+1,1)=.5*(WFACE(I,J,1)+WFACE(I,J,2))
 102  P(I+1,J+1,K1)=.5*(WFACE(I,J,K1)+WFACE(I,J,K0))
C 4th-order-accurate EXCEPT near bottom
      DO 106 K=2,K2
      L=K+1
      DO 104 J=1,J2
      DO 104 I=1,I2
 104  P(I+1,J+1,K)=12.*(WFACE(I,J,K)+WFACE(I,J,L))+
     1 (WFACE(I,J,K)+WFACE(I,J,L)-WFACE(I,J,K-1)-WFACE(I,J,K+2))
C normalization
      DO 106 J=2,J1
      DO 106 I=2,I1
 106  P(I,J,K)=O24*P(I,J,K)

      LB=1
      LT=2
      DO 250 J=1,J2
      DO 250 I=1,I2
      UZ(I,J,1)=0.
      VZ(I,J,1)=0.
 250  TZ(I,J,1)=0.
C =================================
C CALCULATE ALL INTERNAL FLUXES and
C SUBSTITUTE INTO CONSERVATION LAWS
C =================================
C Level-by-level moving window reduces storage.
C Note: all BOUNDARY fluxes are masked out in this moving window loop 500,
C but are  included later. This is the main computation loop.
      DO 500 K=1,K1
      L=K+1

C ---------------------------
C 4th order pressure gradient
C ---------------------------
Calculate face-averaged pressure at sides of each control volume
C UFACE(I1,J2) for U-faces, VFACE(I2,J1) for V-faces
      DO 276 J=1,J2
      DO 275 I=2,I2
 275  UFACE(I,J)=6.*(P(I,J+1,K)+P(I+1,J+1,K))
     1 +IU(I-1,J+1,K)*IU(I+1,J+1,K)*
     2 (P(I,J+1,K)+P(I+1,J+1,K)-P(I-1,J+1,K)-P(I+2,J+1,K))
C use nearest neighbor when data is not available
C may be improved at duo grid boundary using coupled grid values
      DO 276 I=1,I1
 276  UFACE(I,J)=IU(I,J+1,K)*UFACE(I,J)+(1.-IU(I,J+1,K))*
     1 12.*(IN(I,J+1,K)*P(I,J+1,K)+IN(I+1,J+1,K)*P(I+1,J+1,K))

      DO 280 I=1,I2
      DO 279 J=2,J2
 279  VFACE(I,J)=6.*(P(I+1,J,K)+P(I+1,J+1,K))
     1 +IV(I+1,J-1,K)*IV(I+1,J+1,K)*
     2 (P(I+1,J,K)+P(I+1,J+1,K)-P(I+1,J-1,K)-P(I+1,J+2,K))
      DO 280 J=1,J1
 280  VFACE(I,J)=IV(I+1,J,K)*VFACE(I,J)+(1.-IV(I+1,J,K))*
     1 12.*(IN(I+1,J,K)*P(I+1,J,K)+IN(I+1,J+1,K)*P(I+1,J+1,K))

C First pressure differences
      DO 304 J=2,J1
      DO 304 I=2,I1
C set PX=PY=0 everywhere on land
      PX(I,J)=IN(I,J,K)*O12*(UFACE(I,J-1)-UFACE(I-1,J-1))
 304  PY(I,J)=IN(I,J,K)*O12*(VFACE(I-1,J)-VFACE(I-1,J-1))

C ---------------
C Vertical fluxes
C ---------------
      IF (K.NE.K1) GO TO 317
C Set bottom fluxes to zero (drag law applied after label 500)
      DO 308 J=1,J2
      DO 308 I=1,I2
      UZ(I,J,LT)=0.
      VZ(I,J,LT)=0.
 308  TZ(I,J,LT)=0.
      GO TO 359
 317  DO 320 J=2,J1
      DO 320 I=2,I1
      SCR(I,J,1)=6.*(U2(I,J,K)+U2(I,J,L))
      SCR(I,J,2)=6.*(V2(I,J,K)+V2(I,J,L))
 320  SCR(I,J,3)=6.*(T2(I,J,K)+T2(I,J,L))
      IF (K.EQ.1.OR.K.GE.K2) GO TO 340
C     DO 330 J=2,J1
C     DO 330 I=2,I1
C Use lower order near sw corner
      DO 330 J=3,J1
      DO 330 I=3,I1
      TMP=IN(I,J,K-1)*IN(I,J,K)*IN(I,J,L)*IN(I,J,K+2)
      SCR(I,J,1)=SCR(I,J,1)+TMP*
     1 (-ULF(I,J,K-1)+U2(I,J,K)+U2(I,J,L)-U2(I,J,K+2))
      SCR(I,J,2)=SCR(I,J,2)+TMP*
     1 (-VLF(I,J,K-1)+V2(I,J,K)+V2(I,J,L)-V2(I,J,K+2))
 330  SCR(I,J,3)=SCR(I,J,3)+TMP*
     1 (-TLF(I,J,K-1)+T2(I,J,K)+T2(I,J,L)-T2(I,J,K+2))
 340  DO 344 N=1,3
      DO 344 J=2,J1
      DO 344 I=2,I1
 344  SCR(I,J,N)=O12*SCR(I,J,N)
      DO 350 J=2,J1
      DO 350 I=2,I1
      TMP=W(I,J,L)
      UZ(I-1,J-1,LT)=TMP*SCR(I,J,1)
     1 -EV(I-1,J-1,K)*(U1(I,J,L)-U1(I,J,K))*IW(I,J,L)
      VZ(I-1,J-1,LT)=TMP*SCR(I,J,2)
     1 -EV(I-1,J-1,K)*(V1(I,J,L)-V1(I,J,K))*IW(I,J,L)
 350  TZ(I-1,J-1,LT)=TMP*SCR(I,J,3)
     1 -HV(I-1,J-1,K)*(T1(I,J,L)-T1(I,J,K))*IW(I,J,L)

C -------------------
C Longitudinal fluxes
C -------------------
 359  DO 370 J=2,J1
C Higher order interpolations
      DO 361 I=1,I1
      SCR(I,J,1)=6.*(U2(I,J,K)+U2(I+1,J,K))
      SCR(I,J,2)=6.*(V2(I,J,K)+V2(I+1,J,K))
 361  SCR(I,J,3)=6.*(T2(I,J,K)+T2(I+1,J,K))
C     IF (I0.NE.0) GO TO 363
C Skip loop 362 for original 2nd order scheme
      DO 362 I=3,I3
      TMP=IN(I-1,J,K)*IN(I,J,K)*IN(I+1,J,K)*IN(I+2,J,K)
      SCR(I,J,1)=SCR(I,J,1)+TMP*
     1 (-U2(I-1,J,K)+U2(I,J,K)+U2(I+1,J,K)-U2(I+2,J,K))
      SCR(I,J,2)=SCR(I,J,2)+TMP*
     1 (-V2(I-1,J,K)+V2(I,J,K)+V2(I+1,J,K)-V2(I+2,J,K))
 362  SCR(I,J,3)=SCR(I,J,3)+TMP*
     1 (-T2(I-1,J,K)+T2(I,J,K)+T2(I+1,J,K)-T2(I+2,J,K))
 363  DO 364 N=1,3
      DO 364 I=1,I1
 364  SCR(I,J,N)=O12*SCR(I,J,N)
      AM=DM0*ODX(J)
      AH=DE0*ODX(J)
      DO 370 I=1,I1
C UX could be simply U(I,J,K)**2 + ...., but this does not conserve
C energy and is unstable
      UX(I,J-1)=(U(I,J,K)*SCR(I,J,1)
     1 -AM*(U1(I+1,J,K)-U1(I,J,K)))*IU(I,J,K)
      VX(I,J-1)=(U(I,J,K)*SCR(I,J,2)
     1 -AM*(V1(I+1,J,K)-V1(I,J,K)))*IU(I,J,K)
 370  TX(I,J-1)=(U(I,J,K)*SCR(I,J,3)
     1 -AH*(T1(I+1,J,K)-T1(I,J,K)))*IU(I,J,K)

C ------------------
C Latitudinal fluxes
C ------------------
      DO 380 J=1,J1
      DO 371 I=2,I1
      SCR(I,J,1)=6.*(U2(I,J,K)+U2(I,J+1,K))
      SCR(I,J,2)=6.*(V2(I,J,K)+V2(I,J+1,K))
 371  SCR(I,J,3)=6.*(T2(I,J,K)+T2(I,J+1,K))
C     IF (I0.NE.0) GO TO 373
C Skip loop 372 for original 2nd order scheme
      IF (J.EQ.1.OR.J.EQ.J1) GO TO 373
      DO 372 I=2,I1
      TMP=IN(I,J-1,K)*IN(I,J,K)*IN(I,J+1,K)*IN(I,J+2,K)
      SCR(I,J,1)=SCR(I,J,1)+TMP*
     1 (-U2(I,J-1,K)+U2(I,J,K)+U2(I,J+1,K)-U2(I,J+2,K))
      SCR(I,J,2)=SCR(I,J,2)+TMP*
     1 (-V2(I,J-1,K)+V2(I,J,K)+V2(I,J+1,K)-V2(I,J+2,K))
 372  SCR(I,J,3)=SCR(I,J,3)+TMP*
     1 (-T2(I,J-1,K)+T2(I,J,K)+T2(I,J+1,K)-T2(I,J+2,K))
 373  DO 374 N=1,3
      DO 374 I=2,I1
 374  SCR(I,J,N)=O12*SCR(I,J,N)
      AM=DM0*ODYV(J)
      AH=DE0*ODYV(J)
      DO 380 I=2,I1
      UY(I-1,J)=CSV(J)*(V(I,J,K)*SCR(I,J,1)
     1 -AM*(U1(I,J+1,K)-U1(I,J,K)))*IV(I,J,K)
      VY(I-1,J)=CSV(J)*(V(I,J,K)*SCR(I,J,2)
     1 -AM*(V1(I,J+1,K)-V1(I,J,K)))*IV(I,J,K)
 380  TY(I-1,J)=CSV(J)*(V(I,J,K)*SCR(I,J,3)
     1 -AH*(T1(I,J+1,K)-T1(I,J,K)))*IV(I,J,K)
 
C ======================
C CONSERVATION EQUATIONS
C ======================
C Note: These are only partial updates. Pressure, Coriolis and boundary
C forcings are added after statement label 500.
      DO 440 J=2,J1
      ODYJ=OCS(J)*ODY(J)
      DO 440 I=2,I1
      DTIN=DT*IN(I,J,K)
C Longitudinal momentum
      U2(I,J,K)=U1(I,J,K)-DTIN*
     1 ((UX(I,J-1)-UX(I-1,J-1)+PX(I,J))*ODX(J)
     2 +(UY(I-1,J)-UY(I-1,J-1))*ODYJ
     3 +(UZ(I-1,J-1,LT)-UZ(I-1,J-1,LB))*ODZ(K))
C Latitudinal momentum
      V2(I,J,K)=V1(I,J,K)-DTIN*
     1 ((VX(I,J-1)-VX(I-1,J-1))*ODX(J)
     2 +(VY(I-1,J)-VY(I-1,J-1))*ODYJ+PY(I,J)*ODY(J)
     3 +(VZ(I-1,J-1,LT)-VZ(I-1,J-1,LB))*ODZ(K))
C Temperature
 440  T2(I,J,K)=T1(I,J,K)-DTIN*((TX(I,J-1)-TX(I-1,J-1))*ODX(J)
     1 +(TY(I-1,J)-TY(I-1,J-1))*ODYJ
     2 +(TZ(I-1,J-1,LT)-TZ(I-1,J-1,LB))*ODZ(K))
      I=LB
      LB=LT
 500  LT=I

C Wind stress
c     IF (LWIND.NE.0) CALL WIND(U2,V2,DT,ODZ)
 
      IF (LOPEN.EQ.0) GO TO 510
C ========================
C Open boundary conditions
C ========================
 
C These are all determined by "known" normal boundary veloctiy (NBV)
C i.e. boundary normal flux is UPWINDED for both inflow and outflow.
C For inflow, exterior "ghost zone" fields are fluxed inward.  The ghost
C zone fields can be set to climatology.
C Loop 506 below does SCALAR fluxes on boundaries
C Loop 632 below does MOMENTUM fluxes
C Loop 644 below determines NBV
 
C UPWIND METHOD based on NBV.
C Temporarily use ABS() function.
C Later use boundary flag arrays if a vectorized function exists which
C sets an integer array to the sign bit (i.e., 0 OR 1)
C SKIP on first time step because advection velocity divergence in
C    boundary zones has not been cleared out yet.
      TMP=.5*DT
      DO 506 K=1,K1
      DO 504 J=2,J1
C West
      TMPIN=IN(2,J,K)*TMP*ODX(J)
      TEMP=ABS(U(1,J,K))
      TEMP1=TMPIN*(TEMP+U(1,J,K))
      TEMP2=TMPIN*(TEMP-U(1,J,K))
      U2(2,J,K)=U2(2,J,K)+TMPIN*2.*U(1,J,K)**2
      V2(2,J,K)=V2(2,J,K)-TEMP2*VLF(2,J,K)
      T2(2,J,K)=T2(2,J,K)-TEMP2*TLF(2,J,K)+TEMP1*T2(1,J,K)
C East
      TMPIN=IN(I1,J,K)*TMP*ODX(J)
      TEMP=ABS(U(I1,J,K))
      TEMP1=TMPIN*(TEMP+U(I1,J,K))
      TEMP2=TMPIN*(TEMP-U(I1,J,K))
      U2(I1,J,K)=U2(I1,J,K)-TEMP1*ULF(I1,J,K)+TEMP2*U2(I0,J,K)
C No tangential momentum inflow
C     V2(I1,J,K)=V2(I1,J,K)-TEMP1*VLF(I1,J,K)+TEMP2*V2(I0,J,K)
 504  T2(I1,J,K)=T2(I1,J,K)-TEMP1*TLF(I1,J,K)+TEMP2*T2(I0,J,K)
C ==================================================
C NOTE: only a few open points on north boundary !!!
C ==================================================
      DO 506 I=2,I1
C North
      TMPIN=IN(I,J1,K)*TMP*ODY(J1)*CSV(J1)*OCS(J1)
      TEMP=ABS(V(I,J1,K))
      TEMP1=TMPIN*(TEMP+V(I,J1,K))
      TEMP2=TMPIN*(TEMP-V(I,J1,K))
      U2(I,J1,K)=U2(I,J1,K)-TEMP1*ULF(I,J1,K)+TEMP2*U2(I,J0,K)
      V2(I,J1,K)=V2(I,J1,K)-TMPIN*V(I,J1,K)**2
 506  T2(I,J1,K)=T2(I,J1,K)-TEMP1*TLF(I,J1,K)+TEMP2*T2(I,J0,K)
 
C =============
C Bottom stress
C =============
 510  TMP=DT*DRAG
      DO 600 J=2,J1
      DO 600 I=2,I1
      K=KB(I,J)
      DRG=IN(I,J,K)*TMP*ODZ(K)*SQRT(U1(I,J,K)**2+V1(I,J,K)**2)
      U2(I,J,K)=U2(I,J,K)-DRG*U1(I,J,K)
 600  V2(I,J,K)=V2(I,J,K)-DRG*V1(I,J,K)
C Trapezoidal Coriolis
      DO 602 K=1,K1
      DO 602 J=2,J1
      DO 602 I=2,I1
      TMP=.5*F(J-1)*DT
      TEMP=1./(1.+TMP**2)
      QU=U2(I,J,K)+TMP*V1(I,J,K)
      QV=V2(I,J,K)-TMP*U1(I,J,K)
      U2(I,J,K)=TEMP*(QU+TMP*QV)
 602  V2(I,J,K)=TEMP*(QV-TMP*QU)

C =====================================================
C Interpolate "A" grid quantities to "C" grid locations
C =====================================================
C 4th-order interpolations
      DO 635 K=1,K1
      DO 625 J=2,J1
      DO 621 I=2,I2
 621  SCR(I,J,1)=6.*(U2(I,J,K)+U2(I+1,J,K))
C     GO TO 623
C Skip loop 622 for 2nd order scheme
      DO 622 I=3,I3
      TMP=IN(I-1,J,K)*IN(I,J,K)*IN(I+1,J,K)*IN(I+2,J,K)
 622  SCR(I,J,1)=SCR(I,J,1)+TMP*
     1 (-U2(I-1,J,K)+U2(I,J,K)+U2(I+1,J,K)-U2(I+2,J,K))
 623  DO 625 I=2,I2
 625  U(I,J,K)=O12*SCR(I,J,1)*IU(I,J,K)
      DO 631 J=2,J2
      DO 631 I=2,I1
 631  SCR(I,J,1)=6.*(V2(I,J,K)+V2(I,J+1,K))
C     GO TO 633
C Skip loop 632 for 2nd order scheme
      DO 632 J=3,J3
      DO 632 I=2,I1
      TMP=IN(I,J-1,K)*IN(I,J,K)*IN(I,J+1,K)*IN(I,J+2,K)
 632  SCR(I,J,1)=SCR(I,J,1)+TMP*
     1 (-V2(I,J-1,K)+V2(I,J,K)+V2(I,J+1,K)-V2(I,J+2,K))
 633  DO 635 J=2,J2
      DO 635 I=2,I1
 635  V(I,J,K)=O12*SCR(I,J,1)*IV(I,J,K)
      IF (LOPEN.EQ.0) GO TO 650

C Neuman western boundary conditions, except no inflow allowed
      DO 644 K=1,K1
      DO 644 J=2,J1
      U(1,J,K)=MIN(0.,U(2,J,K))
      U1(1,J,K)=U(1,J,K)
 644  U2(1,J,K)=U(1,J,K)
 
C ------------------------------
C Subtract mean boundary outflow as required by incompressibility
C ------------------------------
C Sum is the volume flux EXITing region

      SUM=0.
      DO 5642 K=1,K1
      DO 5641 I=2,I1
 5641 SUM=SUM+V(I,J1,K)*IN(I,J1,K)/(ODYV(J1)*ODZ(K))
      DO 5642 J=2,J1
 5642 SUM=SUM+(U(I1,J,K)*IN(I1,J,K)-U(1,J,K)*IN(2,J,K))/(ODY(J)*ODZ(K))
C Adjust outflows
      SUM=SUM/AROUT
      IF (MOD(ITF,M2).LT.3) WRITE(*,648) SUM
 648  FORMAT('outflow vel correction= ',1PE9.2,' cm/sec')
      DO 649 K=1,K1
      DO 649 J=2,J1
 649  U(1,J,K)=U(1,J,K)+SUM*IN(2,J,K)

C Iterate solver to get zero normal flow at shore
 650  MXMASK=5
C Use SCR and SCR to store changes for incompressibility
      DO 653 K=1,K1
      DO 653 J=1,J0
      DO 653 I=1,I0
 653  SCR(I,J,K)=0.
      ITMASK=0
 655  ITMASK=ITMASK+1
C Zero out advection velocity over land
      TMP=0.
      DO 658 J=2,J1
      DO 658 I=2,I2
 658  U(I,J,1)=IU0(I,J)*U(I,J,1)
      DO 660 J=2,J2
      DO 660 I=2,I1
 660  V(I,J,1)=IV0(I,J)*V(I,J,1)

C ======================================================================
Create non-divergent U,V,W for next step by "clearing out the divergence
C of the barotropic mode"
C ======================================================================
      DO 661 K=1,K1
      TEMP1=1./ODZ(K)
      DO 661 J=2,J1
      TEMP=OCS(J)*ODY(J)
      DO 661 I=2,I1
 661  W(I,J,K+1)=W(I,J,K)-((U(I,J,K)-U(I-1,J,K))*ODX(J)
     1 +(CSV(J)*V(I,J,K)-CSV(J-1)*V(I,J-1,K))*TEMP)*TEMP1
      DO 663 J=2,J1
      DO 663 I=2,I1
 663  S(I-1,J-1)=-W(I,J,KB(I,J)+1)
C For rigid top get top level pressure adjustment from EVP solver and
C barotropically adjust U, V, W
      CALL REP(AL,AB,AC,AR,AT,RINV,RINV1,DUM0,DUM1,DUM2,S,H,X,IE,I0,I2,
     1 NB0)
C Rigid-lid pressure adjustment
      DO 664 J=2,J1
      DO 664 I=2,I1
 664  P0(I,J)=P0(I,J)+ODT*X(I,J)

C Barotropic mode adjustment (from pressure gradient change)
      DO 668 J=2,J1
      DO 668 I=2,I2
 668  SCR(I,J,1)=(X(I+1,J)-X(I,J))*ODX(J)
      DO 669 J=2,J2
      DO 669 I=2,I1
 669  SCR(I,J,2)=(X(I,J+1)-X(I,J))*ODYV(J)
C Top layer
      DO 670 J=2,J1
      DO 670 I=2,I2
      SCR(I,J,3)=SCR(I,J,3)-SCR(I,J,1)
 670  U(I,J,1)=U(I,J,1)-SCR(I,J,1)
      DO 671 J=2,J2
      DO 671 I=2,I1
      SCR(I,J,4)=SCR(I,J,4)-SCR(I,J,2)
 671  V(I,J,1)=V(I,J,1)-SCR(I,J,2)
C Remaining layers
      DO 681 K=2,K1
      DO 680 J=2,J1
      DO 680 I=2,I2
 680  U(I,J,K)=U(I,J,K)-SCR(I,J,1)*IU(I,J,K)
      DO 681 J=2,J2
      DO 681 I=2,I1
 681  V(I,J,K)=V(I,J,K)-SCR(I,J,2)*IV(I,J,K)
 
Check convergence to zero flow over land
      TMP=0.
      DO 682 J=2,J1
      DO 682 I=2,I2
 682  TMP=MAX(TMP,(1.-IU0(I,J))*ABS(U(I,J,1)))
      DO 683 J=2,J2
      DO 683 I=2,I1
 683  TMP=MAX(TMP,(1.-IV0(I,J))*ABS(V(I,J,1)))
      I=ITMASK-1
 684  FORMAT
     1 (9X,'solver iteration #',I2,', max vel on land=',F9.5,' cm/sec')
      IF (TMP.GT..0025.AND.ITMASK.LT.MXMASK) GO TO 655
C This completes the advanced time level advection velocity
C having exactly zero barotropic divergence and satisfying all
C kinematic and specified open boundary conditions
      DO 685 K=1,K1
      TMP=1./ODZ(K)
      DO 685 J=2,J1
      TEMP=OCS(J)*ODY(J)
      DO 685 I=2,I1
 685  W(I,J,K+1)=IW(I,J,K+1)*(W(I,J,K)-((U(I,J,K)-U(I-1,J,K))*ODX(J)
     1 +(CSV(J)*V(I,J,K)-CSV(J-1)*V(I,J-1,K))*TEMP)*TMP)
 
C Interpolate incompressibility induced changes back to cell centers
C High order
      DO 700 K=1,K1
      DO 689 J=2,J1
 
C ====================================================================
C These I-loops need contraction when longitudinal boundaries are open!!!
C    (DO 686 I=3,I2, etc)
C ====================================================================
      DO 686 I=2,I1
 686  SCR(I,J,1)=12.*(IU(I-1,J,K)*SCR(I-1,J,3)+IU(I,J,K)*SCR(I,J,3))
      DO 687 I=3,I2
 687  SCR(I,J,1)=SCR(I,J,1)
     1 -IU(I-2,J,K)*SCR(I-2,J,3)+IU(I-1,J,K)*SCR(I-1,J,3)
     2 +IU(I,J,K)*SCR(I,J,3)-IU(I+1,J,K)*SCR(I+1,J,3)
      DO 688 I=2,I1
 688  U2(I,J,K)=IN(I,J,K)*(U2(I,J,K)+O24*SCR(I,J,1))
 689  CONTINUE
      DO 696 J=3,J2
      DO 696 I=2,I1
 696  SCR(I,J,1)=12.*(IV(I,J-1,K)*SCR(I,J-1,4)+IV(I,J,K)*SCR(I,J,4))
      DO 697 J=4,J3
      DO 697 I=2,I1
 697  SCR(I,J,1)=SCR(I,J,1)
     1 -IV(I,J-2,K)*SCR(I,J-2,4)+IV(I,J-1,K)*SCR(I,J-1,4)
     2 +IV(I,J,K)*SCR(I,J,4)-IV(I,J+1,K)*SCR(I,J+1,4)
      DO 698 J=3,J2
      DO 698 I=2,I1
 698  V2(I,J,K)=IN(I,J,K)*(V2(I,J,K)+O24*SCR(I,J,1))
      DO 699 I=2,I1
      V2(I,2,K)=.5*(V(I,1,K)+V(I,2,K))
 699  V2(I,J1,K)=.5*(V(I,J2,K)+V(I,J1,K))
 700  CONTINUE
 
      IF (ITF-IT0.GT.72.AND.MOD(ITF,72).NE.0) GO TO 712
C =====================
Check incompressibility
C =====================
      TMP=0.
      ERR=0.
      DO 710 K=1,K1
      DO 710 J=2,J1
      DO 710 I=2,I1
      TMP1=(U(I,J,K)-U(I-1,J,K))*ODX(J)
      TMP2=(CSV(J)*V(I,J,K)-CSV(J-1)*V(I,J-1,K))*OCS(J)*ODY(J)
      TMP3=(W(I,J,K+1)-W(I,J,K))*ODZ(K)
      TMP=TMP+MAX(ABS(TMP1),ABS(TMP2),ABS(TMP3))*IN(I,J,K)
 710  ERR=ERR+ABS(TMP1+TMP2+TMP3)*IN(I,J,K)
      ERR=ERR/TMP
      WRITE(*,711) ERR
      WRITE(14,711) ERR
 711  FORMAT(' *** NORMALIZED mean incompressibility error = ',1PE9.2)
 
C ========================
C Update using FLTW method
C ========================
 712  DO 745 K=1,K1
      DO 745 J=2,J1
      DO 745 I=2,I1
      T1(I,J,K)=OFLTW*TLF(I,J,K)+FLTW*(T1(I,J,K)+T2(I,J,K))
      U1(I,J,K)=OFLTW*ULF(I,J,K)+FLTW*(U1(I,J,K)+U2(I,J,K))
      V1(I,J,K)=OFLTW*VLF(I,J,K)+FLTW*(V1(I,J,K)+V2(I,J,K))
      TLF(I,J,K)=T2(I,J,K)
      ULF(I,J,K)=U2(I,J,K)
 745  VLF(I,J,K)=V2(I,J,K)
 
      RETURN
C =============================
C Biharmonic filter application
C to barotropic mode only
C =============================
C This reduces surface noise induced by realistic surface heating and
C wind stress. This is for aesthetics in this model version only.  Noise
C does not affect model robustness or good dynamical behavior for
C resolved scales.
C Each call to FLTX,FLTY multiplies pure 2-grid-interval noise by BF
 
      BF=.95
C Filtering surface pressure has detrimental effects
      CALL SETR(SCR,I0*J0*K1,0.)
      DO 800 K=1,K1
      TMP=1./ODZ(K)
      DO 800 J=2,J1
      DO 800 I=2,I1
      SCR(I,J,2)=SCR(I,J,2)+TMP*IN(I,J,K)*U1(I,J,K)
 800  SCR(I,J,3)=SCR(I,J,3)+TMP*IN(I,J,K)*V1(I,J,K)
      DO 805 J=2,J1
      DO 805 I=2,I1
      TMP=1./Z(2*KB(I,J)+1)
      SCR(I,J,2)=TMP*SCR(I,J,2)
      SCR(I,J,3)=TMP*SCR(I,J,3)
      SCR(I,J,4)=SCR(I,J,2)
 805  SCR(I,J,5)=SCR(I,J,3)
      CALL BFLTXY(BF,SCR(1,1,2),SCR,IU0,IV0)
      CALL BFLTXY(BF,SCR(1,1,3),SCR,IU0,IV0)
      DO 810 J=2,J1
      DO 810 I=2,I1
      SCR(I,J,4)=SCR(I,J,2)-SCR(I,J,4)
 810  SCR(I,J,5)=SCR(I,J,3)-SCR(I,J,5)
      DO 820 K=1,K1
      DO 820 J=2,J1
      DO 820 I=2,I1
      U1(I,J,K)=U1(I,J,K)+SCR(I,J,4)
 820  V1(I,J,K)=V1(I,J,K)+SCR(I,J,5)
      IF (MOD(ITF,5).NE.0) RETURN
      BF=.75
      DO 830 K=1,K1
 830  CALL BFLTXY(BF,T1(1,1,K),SCR,IU(1,1,K),IV(1,1,K))
      END
C ----------------------------------------------------------------------
      SUBROUTINE WIND(U,V,DT,ODZ)
C ----------------------------------------------------------------------
      INCLUDE 'resolution.h'
      PARAMETER(I2=I0-2,J2=J0-2)
      DIMENSION U(I0,J0),V(I0,J0),ODZ(*)
      COMMON/WINDS/TAUX(I2,J2),TAUY(I2,J2)

C The velocity arrays U,V have a perimeter "ghost zone".
C Thus, we set wind forcing only in the interior zones.
C TAUX,TAUY are surface wind stress components.
C TAUX,TAUY units are force per unit area (i.e., energy per unit volume).
C
C     TMP=ODZ(1)/RHO
C All units are cgs, so we use RHO=1.
      TMP=DT*ODZ(1)
      DO 100 J=1,J2
      DO 100 I=1,I2
      U(I+1,J+1)=U(I+1,J+1)+TMP*TAUX(I,J)
 100  V(I+1,J+1)=V(I+1,J+1)+TMP*TAUY(I,J)
      END
C ----------------------------------------------------------------------
      SUBROUTINE PP82
C ----------------------------------------------------------------------
C Vertical mixing coefficients

C APPROACH:
C Set background vertical diffusivities to DMZ0 (O(0.1) cm-cm/sec) and
C add Richardson number based vertical diffusivity plus numerical
C contribution according to vertical cell Reynolds number.

C Vertical cell Re is O(100), except during winter cooling conditions
C and in wind-blown surface mixed layer, as internal waves, which
C dominate W below the SML in the model results during summer, do not
C mix T or S in nature.

C NOTE: one cannot depend on diffusive closure by itself if one wants to
C       model possible contra-diffusive effects

      INCLUDE 'resolution.h'
      PARAMETER(I1=I0-1,I2=I0-2,J1=J0-1,J2=J0-2,K1=K0-1,K2=K0-2)
      INTEGER*2 KB,IU0,IV0,IN,IU,IV,IW
      COMMON RHO(I0,J0,K1),PX(I0,J0),PY(I0,J0),
     1 U1(I0,J0,K1),U2(I0,J0,K1),V1(I0,J0,K1),V2(I0,J0,K1),
     2 T1(I0,J0,K1),T2(I0,J0,K1),P0(I0,J0),P(I0,J0,K1),
     3 ULF(I0,J0,K1),VLF(I0,J0,K1),TLF(I0,J0,K1),
     4 EV(I2,J2,K2),HV(I2,J2,K2),DAVG(K1),F(J2),SUMIN(K1)
      COMMON/CGRID/U(I0,J0,K1),V(I0,J1,K1),W(I0,J0,K0)
      COMMON/CONTRO/
     1 DT,TLZ,B,G,DM0,DE0,DMZ0,RZMX,TAU,DRAG,FLTW,AV,DAODT,TAUDT,
     2 TAUDTN,AROUT,KTRM,M2,M6,MVI,MXIT,ISAV,
     3 MXSAV,LRSTRT,LOPEN,LSURF,LTURB,LWIND,LHEAT,LSAVE,LMOVI
      COMMON/SCA/ODT,PRN,DAYS,ORZMX,OFLTW,ITF,IT0
      COMMON/ZFS/Z(K0+K1),ODZ(K1),ODZW(K0)
      COMMON/WINDMX/VBK(K2),HBK(K2)
      COMMON/BATHY/KB(I0,J0),IU0(I0,J0),IV0(I0,J0),
     1 IN(I0,J0,K1),IU(I0,J0,K1),IV(I0,J1,K1),IW(I0,J0,K0)

C max vertical viscosity (limits Ri-based part of vertical mixing)
      VMAX=100.
      DO 750 K=1,K2
      L=K+1
      DO 750 J=J1,2,-1
      DO 750 I=2,I1
      TEMP=ORZMX*ABS(W(I,J,L))
C Pacanowski and Philander (1981): EVISC=A*(1/(1+B*RI))**N
C EVISC=turbulent eddy viscosity; DIFFUSE=turbulent diffusivity
C     RI=MAX(-0.02D0,980.*(RHO(I,J,L)-RHO(I,J,K))*ODZW(L)/(ODZW(L)**2*
C    1 (0.001D0+(U2(I,J,L)-U2(I,J,K))**2+(V2(I,J,L)-V2(I,J,K))**2)))
C More rapid diffusive adjustment
      RI=MAX(-0.5D0,980.*(RHO(I,J,L)-RHO(I,J,K))*ODZW(L)/(ODZW(L)**2*
     1 (0.001D0+(U2(I,J,L)-U2(I,J,K))**2+(V2(I,J,L)-V2(I,J,K))**2)))
      TMP=1./(1.+RI)
C Reduce cross-pycnocline irreversible diffusion effects
C Kosher because internal waves dominate W at pycnocline
C     TEMP=TEMP*TMP
      TEMP=TEMP*TMP**2
      EVISC=MIN(10.*TMP**2,VMAX)
C     EVISC=MIN(2.5*TMP**2,VMAX)
      DIFFUSE=EVISC*TMP*ODZW(L)
      HV(I-1,J-1,K)=(DIFFUSE+TEMP)*IW(I,J,L)
 750  EV(I-1,J-1,K)=(EVISC*ODZW(L)+TEMP)*IW(I,J,L)
      END
C ******************
C GRAPHICS PACKAGE *
C ******************
C ----------------------------------------------------------------------
      SUBROUTINE FSPLTS(MXIT,NFLG)
C SAVES PLOT DATA
C ----------------------------------------------------------------------
      INCLUDE 'resolution.h'
      PARAMETER(IJ0=I0+J0,I1=I0-1,I2=I0-2,J1=J0-1,J2=J0-2,
     1 K1=K0-1,K2=K0-2)
      INTEGER*2 KB,IU0,IV0,IN,IU,IV,IW
      COMMON RHO(I0,J0,K1),PX(I0,J0),PY(I0,J0),
     1 U1(I0,J0,K1),U2(I0,J0,K1),V1(I0,J0,K1),V2(I0,J0,K1),
     2 T1(I0,J0,K1),T2(I0,J0,K1),P0(I0,J0),P(I0,J0,K1),
     3 ULF(I0,J0,K1),VLF(I0,J0,K1),TLF(I0,J0,K1),
     4 EV(I2,J2,K2),HV(I2,J2,K2),DAVG(K1),F(J2),SUMIN(K1)
      COMMON/TINIT/T0(I0,J0,K1)
      COMMON/CGRID/U(I0,J0,K1),V(I0,J1,K1),W(I0,J0,K0)
      COMMON/KLIMAT/SCR(I0,J0,K1),SC1(IJ0),SC2(IJ0)
      COMMON/SCA/ODT,PRN,DAYS,ORZMX,OFLTW,ITF,IT0
      COMMON/METRICS/
     1 Y(J0),YV(J0),
     2 CS(J0),CSV(J1),OCS(J0),OCSV(J1),
     3 DX(J0),DXV(J1),ODX(J0),ODXV(J1),
     4 DY(J0),DYV(J1),ODY(J0),ODYV(J1)
      COMMON/BATHY/KB(I0,J0),IU0(I0,J0),IV0(I0,J0),
     1 IN(I0,J0,K1),IU(I0,J0,K1),IV(I0,J1,K1),IW(I0,J0,K0)
      COMMON/ZFS/Z(K0+K1),ODZ(K1),ODZW(K0)

C ---------
C X-Y PLOTS
C ---------
      K=1
C sill level
      IF (ITF.NE.MXIT) K=KB(I0/2,J1)
Calculate vorticity at natural (grid line intersect) points
 1    CALL SETR(SCR,2*I0*J0,0.)
      TMP=7.*24.*3600.
      DO 6 J=2,J2
      DO 6 I=2,I2
 6    SCR(I,J,2)=
     1 (V(I+1,J,K)-V(I,J,K))*ODX(J)-(U(I,J+1,K)-U(I,J,K))*ODYV(J)
      DO 8 J=1,J2
      DO 8 I=1,I2
C Interpolate vorticity to cell centers
 8    SCR(I+1,J+1,1)=.25*TMP*
     1 (SCR(I,J,2)+SCR(I+1,J,2)+SCR(I,J+1,2)+SCR(I+1,J+1,2))
      CALL XYPLOT('s','vorticity (per week)          ',SCR,K,K)
      TMP=.5*24.*36.
      DO 15 J=2,J1
      DO 15 I=2,I1
      SCR(I,J,1)=TMP*(W(I,J,K)+W(I,J,K+1))
 15   SCR(I,J,6)=P(I,J,K)/980.
      CALL XYPLOT('T','temperature (degrees c)       ',T2(1,1,K),K,K)
      CALL XYPLOT('P','pressure-equivalent f.s.a.(cm)',SCR(1,1,6),K,K)
      CALL XYPLOT('u','longitudinal velocity (cm/sec)',U2(1,1,K),K,K)
      CALL XYPLOT('v','latitudinal velocity (cm/sec) ',V2(1,1,K),K,K)
      CALL XYPLOT('W','vertical velocity (m/day)     ',SCR,K,K)
      IF (ITF.NE.MXIT) K=2*K
      IF (ITF.EQ.MXIT) K=K+1
      IF (K.LT.K0) GO TO 1

 20   IF (ITF.NE.MXIT) RETURN

C x-average density plume
C from center inflow to 50 km from outflow
      IHI=8.5*I2/11.
      ILO=I2/22.
      CALL SETR(SCR,I0*J0*K1,0.)
C build up average starting at center of inflow
      DO 50 I=IHI,ILO,-1
      DO 40 K=1,K1
      DO 40 J=2,J1
 40   SCR(1,J,K)=SCR(1,J,K)+T0(I,J,K)-T2(I,J,K)
      TMP=1./(IHI-I+1.)
C     TMP=1./(I-ILO+1.)
      DO 41 K=1,K1
      DO 41 J=2,J1
 41   SCR(2,J,K)=TMP*SCR(1,J,K)
 50   IF (I.LT.IHI-5)
     1 CALL YZPLOT('T','downstream zonal avg Tanom    ',SCR(2,1,1),I,I)

C ---------
C X-Z PLOTS
C ---------
      DO 130 J=2,J1
C BD97's sill problem
      IF (MOD(J,10).NE.0.AND.J.LT.J1-4) GO TO 130
      TMP=7.*24.*36.
      TEMP=25.*TMP
      DO 110 K=1,K1
      DO 110 I=2,I1
      SCR(I,4,K)=.5*TMP*(W(I,J,K)+W(I,J,K+1))
 110  SCR(I,5,K)=TEMP*(
     1  (V(I,J-1,K)-V(I-1,J-1,K)+V(I+1,J-1,K)-V(I,J-1,K))*ODXV(J-1)
     2 +(V(I+1,J,K)-V(I,J,K)+V2(I,J,K)-V2(I-1,J,K))*ODXV(J)
     3 -(U(I-1,J,K)-U(I-1,J-1,K)+U(I,J,K)-U(I,J-1,K))*ODYV(J-1)
     4 -(U(I,J+1,K)-U(I,J,K)+U2(I-1,J+1,K)-U2(I-1,J,K))*ODYV(J))
      CALL XZPLOT('T','temperature (degrees C)       ',T2(1,J,1),J,J)
      CALL XZPLOT('U','longitudinal velocity (cm/sec)',U2(1,J,1),J,J)
      CALL XZPLOT('V','latitudinal velocity (cm/sec) ',V2(1,J,1),J,J)
      CALL XZPLOT('w','vertical velocity(meters/week)',SCR(1,4,1),J,J)
      CALL XZPLOT('s','vorticity (per week)          ',SCR(1,5,1),J,J)
 130  CONTINUE

C ---------
C Y-Z PLOTS
C ---------
C BD97 sill problem
      DO 230 I=2,I1
      IF (MOD(I,5).NE.0) GO TO 230
      CALL YZPLOT('T','temperature (degrees c)       ',T2(I,1,1),I,I)
      CALL YZPLOT('U','longitudinal velocity (cm/sec)',U2(I,1,1),I,I)
      CALL YZPLOT('V','latitudinal velocity (cm/sec) ',V2(I,1,1),I,I)
 230  CONTINUE
      END
C ----------------------------------------------------------------------
      SUBROUTINE XYPLOT(FN,FNAME,FLD,KL,KH)
C ----------------------------------------------------------------------
      INCLUDE 'resolution.h'
      PARAMETER(IJ0=I0+J0,I1=I0-1,J1=J0-1,K1=K0-1)
      CHARACTER FN*1,FNAME*30
      INTEGER*2 KB,IU0,IV0,IN,IU,IV,IW,NFLD
      COMMON/KLIMAT/SCR(I0,J0,K1),SC1(IJ0),SC2(IJ0)
      COMMON/FLOAT/NFLD(I0,J0)
      COMMON/BATHY/KB(I0,J0),IU0(I0,J0),IV0(I0,J0),
     1 IN(I0,J0,K1),IU(I0,J0,K1),IV(I0,J1,K1),IW(I0,J0,K0)
      COMMON/SCA/ODT,PRN,DAYS,ORZMX,OFLTW,ITF,IT0
      DIMENSION FLD(I0,J0,1)
      DO 100 K=KL,KH
      KK=K-KL+1
      CALL RANGE(FLD(1,1,KK),SC1,SC2,IN(1,1,K),I0,2,2,I1,J1,
     1 ILO,JLO,IHI,JHI,FMIN,FMAX)
C     WRITE(*,5) FN,K,FMIN,ILO,JLO,FMAX,IHI,JHI
      IF (ITF.LT.100) WRITE(14,5) FN,K,FMIN,ILO,JLO,FMAX,IHI,JHI
 5    FORMAT(A1,' AT LEVEL ',I2,': MIN=',1PE9.2,' AT (',I3,',',I3,
     1 '); MAX=',1PE9.2,' AT (',I3,',',I3,')')
      FRNG=FMAX-FMIN
      RMIN=FMIN+1.E-5*FRNG
      RMAX=FMAX-1.E-5*FRNG
      IF ((RMAX-RMIN).EQ.0.) RETURN
 2    RF=9999./FRNG
C NFLD=9999.*(FLD-RMIN)/FRNG
      DO 11 J=2,J1
      DO 11 I=2,I1
 11   NFLD(I,J)=MIN(9999.,RF*(FLD(I,J,KK)-RMIN))
 100  WRITE(30) FN,FNAME,DAYS,K,ILO,JLO,IHI,JHI,RMIN,RMAX,
     1 ((NFLD(I,J),I=2,I1),J=2,J1)
      END
C ----------------------------------------------------------------------
      SUBROUTINE XZPLOT(FN,FNAME,FLD,JL,JH)
C ----------------------------------------------------------------------
      INCLUDE 'resolution.h'
      PARAMETER(IJ0=I0+J0,I1=I0-1,J1=J0-1,K1=K0-1)
      CHARACTER FN*1,FNAME*30
      INTEGER*2 KB,IU0,IV0,IN,INXZ,IU,IV,IW,NFLD
      COMMON/KLIMAT/SCR(I0,J0,K1),SC1(IJ0),SC2(IJ0)
      COMMON/BATHY/KB(I0,J0),IU0(I0,J0),IV0(I0,J0),
     1 IN(I0,J0,K1),IU(I0,J0,K1),IV(I0,J1,K1),IW(I0,J0,K0)
      COMMON/SCA/ODT,PRN,DAYS,ORZMX,OFLTW,ITF,IT0
      DIMENSION FLD(I0,J0,*),NFLD(I0,K1),XZ(I0,K1),INXZ(I0,K1)
      DO 100 J=JL,JH
      JJ=J-JL+1
      DO 10 K=1,K1
      DO 10 I=2,I1
      XZ(I,K)=FLD(I,JJ,K)
 10   INXZ(I,K)=IN(I,J,K)
      CALL RANGE(XZ,SC1,SC2,INXZ,I0,2,1,I1,K1,ILO,KLO,IHI,KHI,
     1 FMIN,FMAX)
      IF (ITF.LT.100) WRITE(14,5) FN,J,FMIN,FMAX
 5    FORMAT(A1,': MIN,MAX AT LATITUDE ',I2,' = ',2(1X,1PE9.2))
      FRNG=FMAX-FMIN
      RMIN=FMIN+1.E-5*FRNG
      RMAX=FMAX-1.E-5*FRNG
      IF ((RMAX-RMIN).EQ.0.) RETURN
 2    RF=9999./FRNG
C NFLD=9999.*(FLD-RMIN)/FRNG
      DO 11 K=1,K1
      DO 11 I=2,I1
 11   NFLD(I,K)=MIN(9999.,RF*(XZ(I,K)-RMIN)*INXZ(I,K))
 100  WRITE(31) FN,FNAME,DAYS,J,ILO,KLO,IHI,KHI,RMIN,RMAX,
     1 ((NFLD(I,K),I=2,I1),K=1,K1)
      END
C ----------------------------------------------------------------------
      SUBROUTINE YZPLOT(FN,FNAME,FLD,IL,IH)
C ----------------------------------------------------------------------
      INCLUDE 'resolution.h'
      PARAMETER(IJ0=I0+J0,I1=I0-1,J1=J0-1,K1=K0-1)
      CHARACTER FN*1,FNAME*30
      INTEGER*2 KB,IU0,IV0,IN,INYZ,IU,IV,IW,NFLD
      COMMON/KLIMAT/SCR(I0,J0,K1),SC1(IJ0),SC2(IJ0)
      COMMON/BATHY/KB(I0,J0),IU0(I0,J0),IV0(I0,J0),
     1 IN(I0,J0,K1),IU(I0,J0,K1),IV(I0,J1,K1),IW(I0,J0,K0)
      COMMON/SCA/ODT,PRN,DAYS,ORZMX,OFLTW,ITF,IT0
      DIMENSION FLD(I0,J0,*),NFLD(J0,K1),YZ(J0,K1),INYZ(J0,K1)
      DO 100 I=IL,IH
      II=I-IL+1
      DO 10 K=1,K1
      DO 10 J=2,J1
      YZ(J,K)=FLD(II,J,K)
 10   INYZ(J,K)=IN(I,J,K)
C
C For plotting zonal averages (indicated by I=1)
      IF (I.NE.1) GO TO 20
      DO 15 K=1,K1
      DO 15 J=2,J1
      N=0
      DO 14 III=2,I1
 14   N=N+IN(III,J,K)
      INYZ(J,K)=0
 15   INYZ(J,K)=MIN(1,N)
C
 20   CALL RANGE(YZ,SC1,SC2,INYZ,J0,2,1,J1,K1,JLO,KLO,JHI,KHI,
     1 FMIN,FMAX)
      IF (ITF.LT.100) WRITE(14,21) FN,I,FMIN,FMAX
 21   FORMAT(A1,': MIN,MAX AT LONGITUDE ',I3,' = ',2(1X,1PE9.2))
      FRNG=FMAX-FMIN
      RMIN=FMIN+1.E-5*FRNG
      RMAX=FMAX-1.E-5*FRNG
      IF ((RMAX-RMIN).EQ.0.) RETURN
      RF=9999./FRNG
C NFLD=9999.*(FLD-RMIN)/FRNG
      DO 30 K=1,K1
      DO 30 J=2,J1
 30   NFLD(J,K)=MIN(9999.,RF*(YZ(J,K)-RMIN)*INYZ(J,K))
 100  WRITE(32) FN,FNAME,DAYS,I,JLO,KLO,JHI,KHI,RMIN,RMAX,
     1 ((NFLD(J,K),J=2,J1),K=1,K1)
      END
C ----------------------------------------------------------------------
      SUBROUTINE RANGE
     1 (FLD,SEGMN,SEGMX,IN,IR,IL,JL,IH,JH,ILO,JLO,IHI,JHI,FMIN,FMAX)
C VECTORIZED SEARCH FOR MAX'S AND MIN'S
C ----------------------------------------------------------------------
      INTEGER*2 IN
      DIMENSION FLD(IR,*),SEGMN(*),SEGMX(*),IN(IR,*)
      LBLK=1
      LAB=1
      IHM=IH-1
C ----------------
C SEARCH FOR MAXES
C ----------------
C
C Initialize column maxes
      DO 2 I=IL,IH
 2    SEGMX(I)=-1.E30
      IF (LBLK.EQ.0) GO TO 9
C
C Search for column maxes
C with islands
      DO 5 J=JL,JH
      DO 5 I=IL,IH
 5    SEGMX(I)=(1.-IN(I,J))*SEGMX(I)+IN(I,J)*MAX(SEGMX(I),FLD(I,J))
      GO TO 15
C without islands
 9    DO 10 J=JL,JH
      DO 10 I=IL,IH
 10   SEGMX(I)=MAX(SEGMX(I),FLD(I,J))
C
C Search for max of column maxes
 15   FMAX=SEGMX(IL)
      DO 20 I=IL,IHM
 20   FMAX=MAX(FMAX,SEGMX(I+1))
      IF (LAB.EQ.0) GO TO 50
C
C Search for location of max
      DO 29 I=IL,IH
 29   IF (SEGMX(I).EQ.FMAX) IHI=I
      DO 30 J=JL,JH
 30   IF (FLD(IHI,J).EQ.FMAX) JHI=J
C ---------------
C SEARCH FOR MINS
C ---------------
 50   DO 52 I=IL,IH
 52   SEGMN(I)=1.E30
      IF (LBLK.EQ.0) GO TO 59
      DO 55 J=JL,JH
      DO 55 I=IL,IH
 55   SEGMN(I)=(1.-IN(I,J))*SEGMN(I)+IN(I,J)*MIN(SEGMN(I),FLD(I,J))
      GO TO 65
 59   DO 60 J=JL,JH
      DO 60 I=IL,IH
 60   SEGMN(I)=MIN(SEGMN(I),FLD(I,J))
 65   FMIN=SEGMN(IL)
      DO 70 I=IL,IHM
 70   FMIN=MIN(FMIN,SEGMN(I+1))
      IF (LAB.EQ.0) RETURN
      DO 79 I=IL,IH
 79   IF (SEGMN(I).EQ.FMIN) ILO=I
      DO 80 J=JL,JH
 80   IF (FLD(ILO,J).EQ.FMIN) JLO=J
      END
C ----------------------------------------------------------------------
      SUBROUTINE BFLTXY(PWT,P,S,IU,IV)
C BIHARMONIC FILTER
C ----------------------------------------------------------------------
      INCLUDE 'resolution.h'
      PARAMETER(I1=I0-1,I2=I0-2,I3=I0-2,J1=J0-1,J2=J0-2,J3=J0-3)
      INTEGER*2 IU(I0,*),IV(I0,*)
      DIMENSION P(I0,*),S(I0,*)
      TMP=(1.-PWT)/16.
      DO 20 J=2,J1
      DO 20 I=3,I2
 20   S(I,J)=P(I-1,J)+P(I+1,J)-2.*P(I,J)
      DO 40 J=2,J1
      DO 40 I=4,I3
      TEMP=IU(I-2,J)*IU(I-1,J)*IU(I,J)*IU(I+1,J)
 40   P(I,J)=P(I,J)-TMP*TEMP*(S(I-1,J)+S(I+1,J)-2.*S(I,J))
      DO 60 I=2,I1
      DO 60 J=3,J2
 60   S(I,J)=P(I,J-1)+P(I,J+1)-2.*P(I,J)
      DO 80 J=4,J3
      DO 80 I=2,I1
      TEMP=IV(I,J-2)*IV(I,J-1)*IV(I,J)*IV(I,J+1)
 80   P(I,J)=P(I,J)-TMP*TEMP*(S(I,J-1)+S(I,J+1)-2.*S(I,J))
      END
C ----------------------------------------------------------------------
      SUBROUTINE SETR(A,N,R)
C ----------------------------------------------------------------------
      DIMENSION A(1)
      DO 10 I=1,N
 10   A(I)=R
      END
C *****************
C ELLIPTIC SOLVER *
C *****************
C ----------------------------------------------------------------------
      SUBROUTINE REP(AX,AY,BB,CX,CY,RINV,RINV1,DUM0,DUM1,DUM2,F,H,X,IE,
     1 I0,I2,NBLK)
C ----------------------------------------------------------------------
      REAL*8 RINV,RINV1,DUM0,DUM1,DUM2,X,H
      DIMENSION AX(I2,1),AY(I2,1),BB(I2,1),CX(I2,1),CY(I2,1),
     1 RINV(I2,I2,1),RINV1(I2,I2,1),H(I0,1),IE(1),DUM0(I2,1),DUM1(1),
     2 DUM2(1),F(I2,1),X(I0,1)
      JS=1
      DO 150 NB=1,NBLK
      JF=IE(NB)-2
      DO 105 J=JS,JF
      DO 105 I=1,I2
 105  X(I+1,J+2)=(F(I,J)-AX(I,J)*X(I,J+1)-AY(I,J)*X(I+1,J)-BB(I,J)*
     1 X(I+1,J+1)-CX(I,J)*X(I+2,J+1))/CY(I,J)
      IF (NB.EQ.NBLK) GO TO 150
      J=IE(NB)-1
      DO 115 I=1,I2
 115  DUM1(I)=F(I,J)-AX(I,J)*X(I,J+1)-AY(I,J)*X(I+1,J)-BB(I,J)*
     1 X(I+1,J+1)-CX(I,J)*X(I+2,J+1)-CY(I,J)*X(I+1,J+2)
      J=IE(NB)
      DO 120 N=1,I2
      DUM2(N)=0.
      DO 118 M=1,I2
 118  DUM2(N)=DUM2(N)+DUM1(M)*RINV1(M,N,NB)
      DUM0(N,NB)=X(N+1,J)
 120  X(N+1,J)=X(N+1,J)-DUM2(N)
 150  JS=IE(NB)
      DO 300 NBS=1,NBLK
      NB=NBLK-NBS+1
      JS=1
      IF (NB.NE.1) JS=IE(NB-1)
      JF=IE(NB)-2
      IF (NB.EQ.NBLK) GO TO 201
      J=IE(NB)
      DO 200 N=1,I2
 200  X(N+1,J)=DUM0(N,NB)
 201  N=IE(NB)
      DO 202 J=JS,N
      DO 202 I=1,I0
 202  H(I,J)=0.
      J=IE(NB)-1
      DO 210 I=1,I2
 210  DUM1(I)=F(I,J)-AX(I,J)*X(I,J+1)-AY(I,J)*X(I+1,J)-BB(I,J)*
     1 X(I+1,J+1)-CX(I,J)*X(I+2,J+1)-CY(I,J)*X(I+1,J+2)
      DO 220 N=1,I2
      DUM2(N)=0.
      DO 218 M=1,I2
 218  DUM2(N)=DUM2(N)+DUM1(M)*RINV(M,N,NB)
      H(N+1,JS+1)=DUM2(N)
 220  X(N+1,JS+1)=X(N+1,JS+1)+DUM2(N)
      IF (NB.EQ.1) GO TO 250
      DO 230 M=1,I2
 230  DUM1(M)=H(M+1,JS+1)*CY(M,JS-1)
      J=IE(NB-1)
      DO 240 N=1,I2
      DUM2(N)=0.
      DO 238 M=1,I2
 238  DUM2(N)=DUM2(N)+DUM1(M)*RINV1(M,N,NB-1)
 240  H(N+1,J)=DUM2(N)
 250  DO 300 J=JS,JF
      DO 299 I=1,I2
      H(I+1,J+2)=(-AX(I,J)*H(I,J+1)-AY(I,J)*H(I+1,J)-BB(I,J)*
     1 H(I+1,J+1)-CX(I,J)*H(I+2,J+1))/CY(I,J)
 299  X(I+1,J+2)=X(I+1,J+2)+H(I+1,J+2)
 300  CONTINUE
      END
C ****************
C INITIALIZATION *
C ****************
C ----------------------------------------------------------------------
      SUBROUTINE INITFS
C INITIALIZE ALL CONTROLLING ARRAYS AND DERIVED SCALARS FOR FS
C ----------------------------------------------------------------------
      INCLUDE 'resolution.h'
      PARAMETER(IJ0=I0+J0,I1=I0-1,I2=I0-2,J1=J0-1,J2=J0-2,
     1 K1=K0-1,K2=K0-2,NB1=NB0-1)
      REAL*8 RINV,RINV1,DUM0,DUM1,DUM2,X,H
      CHARACTER CASE*5,DSCRIB*63
      INTEGER*2 KB,IU0,IV0,IN,IU,IV,IW
      COMMON RHO(I0,J0,K1),PX(I0,J0),PY(I0,J0),
     1 U1(I0,J0,K1),U2(I0,J0,K1),V1(I0,J0,K1),V2(I0,J0,K1),
     2 T1(I0,J0,K1),T2(I0,J0,K1),P0(I0,J0),P(I0,J0,K1),
     3 ULF(I0,J0,K1),VLF(I0,J0,K1),TLF(I0,J0,K1),
     4 EV(I2,J2,K2),HV(I2,J2,K2),DAVG(K1),F(J2),SUMIN(K1)
      COMMON/CGRID/U(I0,J0,K1),V(I0,J1,K1),W(I0,J0,K0)
      COMMON/CONTRO/
     1 DT,TLZ,B,G,DM0,DE0,DMZ0,RZMX,TAU,DRAG,FLTW,AV,DAODT,TAUDT,
     2 TAUDTN,AROUT,KTRM,M2,M6,MVI,MXIT,ISAV,
     3 MXSAV,LRSTRT,LOPEN,LSURF,LTURB,LWIND,LHEAT,LSAVE,LMOVI
      COMMON/WINDS/TAUX(I2,J2),TAUY(I2,J2)
      COMMON/KLIMAT/SCR(I0,J0,K1),SC1(IJ0),SC2(IJ0)
      COMMON/SURF/TSURF(I2,J2)
      COMMON/SCA/ODT,PRN,DAYS,ORZMX,OFLTW,ITF,IT0
      COMMON/METRICS/
     1 Y(J0),YV(J0),
     2 CS(J0),CSV(J1),OCS(J0),OCSV(J1),
     3 DX(J0),DXV(J1),ODX(J0),ODXV(J1),
     4 DY(J0),DYV(J1),ODY(J0),ODYV(J1)
      COMMON/SEVP/RINV(I2,I2,NB0),RINV1(I2,I2,NB1),DUM0(I2,NB1),
     1 DUM1(I2),DUM2(I2),X(I0,J0),H(I0,J0),AL(I2,J2),AB(I2,J2),
     2 AC(I2,J2),AR(I2,J2),AT(I2,J2),S(I2,J2),IE(NB0)
      COMMON/BATHY/KB(I0,J0),IU0(I0,J0),IV0(I0,J0),
     1 IN(I0,J0,K1),IU(I0,J0,K1),IV(I0,J1,K1),IW(I0,J0,K0)
      COMMON/ZFS/Z(K0+K1),ODZ(K1),ODZW(K0)
      COMMON/TITLE/CASE,DSCRIB
 
 1    FORMAT('CASE ',A5,' (',A63,')'/'----------'/
     1 ' LRSTRT   MXIT   I0   J0   K0 KTRM LSAVE'/2I7,5I5//
     2 8X,'DT',6X,'FLTW',7X,'TLZ'/1P,3(1X,E9.2)//
     3 7X,'TAU',9X,'G',7X,'DM0',7X,'DE0',6X,'DMZ0',5X,
     4 'RZMX',6X,'DRAG'/1P,8(1X,E9.2))
 2    FORMAT(
     1 /'((KB(I,J),I=2,I1),J=J1,2,-1) (INTERIOR LOGICAL DEPTH ARRAY)'
     2 /(20I4))
 6    FORMAT(2I4/4F13.7,A10)
 8    FORMAT('PSI(SV)'/(10(1X,F7.3)))
 9    FORMAT(5E15.7)
C **************************************
C *** OPEN AND READ INPUT DATA FILES ***
C **************************************
 
C -----------------------------------
C SCALAR DATA DEFINING RUN TO BE MADE
C -----------------------------------
      OPEN(9,file='PREP/DATA/RUNDATA',form='unformatted')
      REWIND 9

      READ(9) CASE,DSCRIB,DT,TLZ,B,G,DM0,DE0,DMZ0,EV,HV,F,
     1 RZMX,TAU,TAUN,DRAG,FLTW,DAODT,ODT,
     2 PRN,ORZMX,OFLTW,Y,YV,CS,CSV,OCS,OCSV,DX,DXV,
     3 ODX,ODXV,DY,DYV,ODY,ODYV,KTRM,LRSTRT,MXIT,ISAV,MXSAV,M2,M6,LOPEN,
     4 LSURF,LTURB,LWIND,LHEAT,LSAVE,LMOVI,MVI

C -----------------------------
C SURFACE TEMPERATURE RESTORING
C AND WIND STRESS DATA
C -----------------------------
C When LWIND=0, we still have TAUX,TAUY written as place holders
C TSURF is needed to avoid overwrites of new restoring data
C when reading restart
c     OPEN(71,file='PREP/DATA/surface',form='unformatted')
c     REWIND 71
c     READ(71) TSURF,TSURF,TAUX,TAUY
c     IF (LWIND.EQ.0) GO TO 15
C Convert from Newtons/m-m to dynes/cm-cm
c     DO 14 J=1,J2
c     DO 14 I=1,I2
c     TAUX(I,J)=10.*TAUX(I,J)
c14   TAUY(I,J)=10.*TAUY(I,J)

C -------------------------------------
C METRICS, Z-LEVELS(Z), AND BATHYMMETRY
C (LOGICAL DEPTH ARRAY (KB)) DATA
C -------------------------------------
 15   OPEN(60,file='PREP/DATA/ZKB',form='unformatted')
      REWIND 60
      READ(60) TLZ,Z,ODZ,ODZW,KB,IN,IU,IV,IW,IU0,IV0
C DON'T CLOSE(60) YET
 
C ----------------
C INITIAL T-S DATA
C ----------------
      OPEN(70,file='PREP/DATA/initial',form='unformatted')
      REWIND 70
C Read restoring surface T and S arrays
C (The first T1 is a place holder for S1 data not used)
      READ(70) T1,T1
      CLOSE(70)
 
C -------------------------
C EVP ELLIPTIC SOLVER DATA
C USED FOR SURFACE PRESSURE
C -------------------------
      OPEN(99,file='PREP/DATA/EVP',form='unformatted')
      REWIND 99
      READ(99) AL,AR,AB,AT,AC,RINV,RINV1,IE
      CLOSE(99)
C **************************
C *** NO MORE INPUT DATA ***
C **************************

C Write formatted run description and parameters
      WRITE(14,1) CASE,DSCRIB,LRSTRT,MXIT,I0,J0,K0,KTRM,LSAVE,DT,FLTW,
     1 TLZ,TAU,G,DM0,DE0,DMZ0,RZMX,DRAG
      DO 21 K=1,K1
      SUMIN(K)=0.
      DO 20 J=2,J1
      DO 20 I=2,I1
 20   SUMIN(K)=SUMIN(K)+IN(I,J,K)
 21   SUMIN(K)=1./SUMIN(K)
      FLTW=.5*FLTW
      TAUDT=1./(TAU*DAODT)
      TAUDTN=1./(TAUN*DAODT)
      DO 28 K=1,K1
      DO 28 J=1,J0
      DO 28 I=1,I0
      TLF(I,J,K)=T1(I,J,K)
 28   T2(I,J,K)=T1(I,J,K)
      WRITE(30) CASE,DSCRIB,LRSTRT,MXIT,I0,J0,K0,IN,ODX,ODY,ODZ,Z
      WRITE(31) CASE,DSCRIB,LRSTRT,MXIT,I0,J0,K0,IN,ODX,ODY,ODZ,Z
      WRITE(32) CASE,DSCRIB,LRSTRT,MXIT,I0,J0,K0,IN,ODX,ODY,ODZ,Z

C -----------------
C READ RESTART DATA
C -----------------
      IF (LRSTRT.EQ.0) GO TO 170
      READ(19) ITF,DAYS,AV,(X(I,2),I=2,I1),U1,U2,V1,V2,
     1 T1,T2,P,ULF,VLF,TLF,U,V,W,EV,HV
 
      WRITE(*,92) ITF,DAYS
      WRITE(14,92) ITF,DAYS
 92   FORMAT('RESTART DATA READ FOR TIME STEP ',I6,', DAY ',F8.2)
      I=DAYS*DAODT+.5
      IF (I.EQ.ITF) GO TO 99
      WRITE(*,93) ITF,I
      WRITE(14,93) ITF,I
 93   FORMAT(/'Time step # changed from',I7,' to',I7,
     1 ' at restart to reflect DT change')
      ITF=I
 99   CONTINUE
C -----------------------------
C OPEN BOUNDARY NORMAL VELOCITY
C -----------------------------
 170  IF (LOPEN.EQ.0) GO TO 430
      OPEN(62,file='PREP/DATA/boundaries',form='unformatted')
      REWIND 62
C -----------------------
C This stuff must be read, whether or not there's a restart!
C -----------------------
      READ(62) AROUT
      IF (LRSTRT.EQ.1) GO TO 415
      READ(62)
     1 ((U(I1,J,K),J=1,J0),K=1,K1),((U2(I0,J,K),J=1,J0),K=1,K1),
     2 ((U(1,J,K),J=1,J0),K=1,K1),((U2(1,J,K),J=1,J0),K=1,K1),
     3 ((V(I,J1,K),I=1,I0),K=1,K1),((V2(I,J0,K),I=1,I0),K=1,K1),
     4 ((V(I,1,K),I=1,I0),K=1,K1),((V2(I,1,K),I=1,I0),K=1,K1)
 415  CLOSE(62)
C Depth
 430  READ(60) ((SCR(I,J,1),I=1,I0),J=1,J0)
      CLOSE(60)
      DO 438 J=2,J1
      DO 438 I=2,I1
C Convert to kilometers
 438  SCR(I,J,1)=1.E-5*SCR(I,J,1)
      CALL XYPLOT('Z','depth (kilometers)            ',SCR,1,1)
 
C -----------------------------------------
C SHOW INITIAL FIELDS WHEN STARTING NEW RUN
C -----------------------------------------
c     K=2
c     IF (LRSTRT.NE.1) K=K1
c     CALL XYPLOT('T','initial temperature(degrees c)',T2,2,K)
c Wind stress
c     DO 450 J=2,J1
c     DO 450 I=2,I1
c     SCR(I,J,4)=TAUX(I-1,J-1)
c     SCR(I,J,5)=TAUY(I-1,J-1)
c450  SCR(I,J,3)=SQRT(TAUX(I-1,J-1)**2+TAUY(I-1,J-1)**2)
c     CALL XYPLOT('M','wind stress magnitude         ',SCR(1,1,3),1,1)
c     CALL XYPLOT('U','long. wind stress(dynes/cm-cm)',SCR(1,1,4),1,1)
c     CALL XYPLOT('V','lat. wind stress (dynes/cm-cm)',SCR(1,1,5),1,1)
      WRITE(14,460) (F(J),J=J2,1,-1)
 460  FORMAT(
     1 /'F(J),J=J2,1,-1) (Latitudinal profile of Coriolis parameter)'
     2 /1P,(8(1X,E9.2)))
      WRITE(14,461) CS
 461  FORMAT('METRICS'/'COSLAT at cell centers'/0P,(10(1X,F5.3)))
      WRITE(14,462) CSV
 462  FORMAT('COSLAT at grid lines'/0P,(10(1X,F5.3)))
      WRITE(14,463) DX
 463  FORMAT('DX at cell centers'/1P,(8(1X,E9.2)))
      WRITE(14,464) DXV
 464  FORMAT('DXV at grid lines'/1P,(8(1X,E9.2)))
      WRITE(14,480) (Z(2*K-1),K=1,K0)
 480  FORMAT(/'Interface depths (meters)',-2P/(10(1X,F7.2)))
      WRITE(14,482) (Z(2*K),K=1,K1)
 482  FORMAT(/'Cell center depths (meters)',-2P/(10(1X,F7.2)))
      WRITE(14,490) (ODZ(K),K=1,K1)
 490  FORMAT(/'Inverse grid increments at cell centers (1/cm)'/
     1 (8(1X,1PE9.2)))
      WRITE(14,492) (ODZW(K),K=1,K0)
 492  FORMAT(/'Inverse grid increments at cell interfaces (1/cm)'/
     1 (8(1X,1PE9.2)))
  
C Bathymmetry
      WRITE(14,2) ((KB(I,J),I=2,I1),J=J1,2,-1)
      WRITE(14,540)
 540  FORMAT(/'Surface layer masking array')
      DO 542 J=J0,1,-1
 542  WRITE(14,543) (IN(I,J,1),I=2,I1)
 543  FORMAT(161I1)
      END
